//
//  ZGPlayGameViewController.m
//  ZegoMiniGameEngineDemo
//
//  Created by zego on 2023/2/16.
//

#import "ZGPlayGameViewController.h"
#import "ZegoRechargeAlterView.h"
#import "BackendService.h"
#import "ZGUserInfoRandom.h"

@interface ZGPlayGameViewController ()<ZegoGameEngineHandler,ZegoEventHandler>

// 游戏加载中。。。label
@property (nonatomic, strong) UILabel *loadingLabel;
// 关闭游戏按钮
@property (nonatomic, strong) UIButton *closeGameBtn;
// 是否要退出
@property (nonatomic, assign) BOOL isBack;

@property (nonatomic, strong) UIView *gameContainView;
@end

@implementation ZGPlayGameViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    [self setInit];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(handleDeviceOrientationChange) name:UIDeviceOrientationDidChangeNotification object:nil];
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    self.tabBarController.hidesBottomBarWhenPushed = YES;
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    self.isBack = YES;
    if (!ZGStringIsEmpty(self.currentGameID)) {
        [ZegoMiniGameEngine.sharedEngine unloadGame:YES];
        // 这里需执行onGameQuit，不能等onUnloadGame回调
        [self onGameQuit];
    }
}

- (void)setInit {
    self.isBack = NO;
//    self.view.backgroundColor = [UIColor blackColor];
    [self.view addSubview:self.gameContainView];
    [self.view addSubview:self.loadingLabel];
    [self.view addSubview:self.closeGameBtn];
    [self.gameContainView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.view.mas_left);
        make.right.mas_equalTo(self.view.mas_right);
        make.bottom.mas_equalTo(self.view.mas_bottom);
        make.top.mas_equalTo(self.view.mas_top).offset(TOP_BAR_HEIGHT);
    }];
    [self.loadingLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(self.view);
        make.centerY.equalTo(self.view);
    }];
    
    [self.closeGameBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.mas_equalTo(self.view).offset(-5);
        make.bottom.equalTo(self.view).offset(-kTLBottomStatusBarHeight);
        make.height.mas_equalTo(36);
        make.width.mas_equalTo(65);
    }];
    
    [ZegoMiniGameEngine.sharedEngine setGameEngineHandler:self];
    [ZegoMiniGameEngine.sharedEngine setGameContainer:self.gameContainView];
    
    if(self.preload) {
        @weakify(self);
        [ZegoMiniGameEngine.sharedEngine preLoadGame:self.currentGameID callBack:^(NSInteger errorCode, int progress) {
            @strongify(self);
            if (errorCode == 0 && progress == 100) {
                self.loadingLabel.hidden = NO;
                NSLog(@"preLoadGame success");
                self.loadingLabel.text = [NSString stringWithFormat:@"%@:%%%d",TLLocalizedString(loading_game), progress];
                [self loadgame];
            } else if (errorCode == 0 && progress != 100) {
                self.loadingLabel.hidden = NO;
                self.loadingLabel.text = [NSString stringWithFormat:@"%@:%%%d",TLLocalizedString(loading_game), progress];
                NSLog(@"preLoadGame progress:%d", progress);
    //            [MBProgressHUD showMessage:[NSString stringWithFormat:@"progress:%f",progress]];
            } else {
                NSLog(@"preLoadGame fail %ld", errorCode);
                [MBProgressHUD showSuccess:[NSString stringWithFormat:@"errorcode:%d",errorCode] withFinishBlock:nil];
            }
        }];
    } else {
        [self loadgame];
    }
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(_refreshPoints) name:@"updateGameData" object:nil];
}

- (void)loadgame {
    // 加载游戏
    NSDictionary *gameConfig = @{
        @"roomID" : self.viewModel.roomID? :@"",
        @"customPlaySound" : @(false),
        @"minGamePoint" : @(0),
        @"useRobot" : @(true),
        @"specificConfig" : @{@"paddingTop" : @"100",
                              @"paddingBottom" : @"0",
//                              @"buttonSelect" : @(0),
//                              @"showShopButton" : @(0),
//                              @"needQuitEvent" : @(1),
        }
    };
    @weakify(self);
    [ZegoMiniGameEngine.sharedEngine loadGame:self.currentGameID
                                     gameMode:ZegoGameModeMallGame
                                   gameConfig:gameConfig
                                     callBack:^(NSInteger errorCode) {
        @strongify(self);
        dispatch_async(dispatch_get_main_queue(), ^{
            if (ZegoMiniGameEngineSuccess == errorCode) {
                [self onGameStart];
                [self.view bringSubviewToFront:self.loadingLabel];
                [self.view bringSubviewToFront:self.closeGameBtn];
            } else {
                [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(load_game_error), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
            }
        });
    }];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(_refreshPoints) name:@"updateGameData" object:nil];
}

- (void)_refreshPoints{
    // 当上个游戏退出后，快速进新的游戏。有可能兑出积分的操作执行很慢导致这时才收到刷新通知。
    NSString *gameID = self.currentGameID;
    if (!ZGStringIsEmpty(gameID)) {
        DemoGameInfo *demoGameInfo = [self.viewModel getDemoGameInfoWithGameID:gameID];
        if(demoGameInfo != nil && demoGameInfo.gamePoint > 0) {
            long tempPoint =  demoGameInfo.gamePoint;
            demoGameInfo.gamePoint = 0;
            // 兑换游戏积分
        }
    }
}

/**
 * 在游戏开始时把业务积分兑换成游戏积分，保证除了游戏中，其他都是自己在维护业务积分
 */
- (void)onGameStart {
    self.loadingLabel.hidden = NO;
    NSString *gameID = self.currentGameID;
    if (!ZGStringIsEmpty(gameID)) {
        DemoGameInfo *demoGameInfo = [self.viewModel getDemoGameInfoWithGameID:gameID];
        if (!ZGObjectIsEmpty(demoGameInfo) && demoGameInfo.gamePoint != 0) {
            long  tempPoint = demoGameInfo.gamePoint;
            demoGameInfo.gamePoint = 0;
            // 兑换游戏积分
        }

    } else {
        [MBProgressHUD showSuccess:TLLocalizedString(game_id_is_empty) withFinishBlock:nil];
    }
}

/**
 * 在游戏退出时把游戏积分兑换成业务积分，保证除了游戏中，其他都是自己在维护业务积分
 */
- (void)onGameQuit {
    NSString *gameID = self.currentGameID;
    DemoGameInfo *demoGameInfo = [self.viewModel getDemoGameInfoWithGameID:gameID];
    if (!ZGObjectIsEmpty(demoGameInfo)) {
        // 兑换游戏积分
    }
}


#pragma mark - action

- (void)onClickCloseGameAction:(UIButton *)sender {
    if (!ZGStringIsEmpty(self.currentGameID)) {
        [ZegoMiniGameEngine.sharedEngine unloadGame:YES];
        [self onGameQuit];
        self.currentGameID = @"";
    }
}

#pragma mark----ZegoGameEngineHandler

- (void)onTokenWillExpire {
    [MBProgressHUD showSuccess:@"onTokenWillExpire....." withFinishBlock:nil];
    [BackendService getTokenWithUserID:self.viewModel.userID appID:KeyCenter.appID completion:^(NSInteger errorCode, NSString * _Nonnull token) {
        if (errorCode == 0 && token.length) {
            [ZegoMiniGameEngine.sharedEngine updateToken:token];
        }
    }];
}

- (void)onGameLoadStateUpdate:(ZegoGameLoadState)gameLoadState {
    if (gameLoadState == ZegoGameLoaded || gameLoadState == ZegoGameLoading) {
        self.loadingLabel.hidden = YES;
    }
}

- (void)onGameStateUpdate:(ZegoGameState)gameState {
    kLog(@"onGameStateUpdate：%ld", gameState);
}

- (void)onGameStateUpdate:(ZegoGameState)gameState reasonCode:(int)reasonCode {
    kLog(@"onGameStateUpdate：%ld reasonCode：%d", gameState,reasonCode);
}

- (void)onPlayerStateUpdate:(ZegoGamePlayerState)playerState {
    [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(update_player_status), playerState] withFinishBlock:nil];
}

- (void)onPointsInsufficient:(NSString *)gameID {
    ZegoRechargeAlterView *alertView = [ZegoRechargeAlterView alertView];
    alertView.pointsAmountLabel.text = [NSString stringWithFormat:@"%d", (int)self.viewModel.businessPoint];
    __weak typeof(ZegoRechargeAlterView) *weakAlertView = alertView;
    [alertView onRechargeBlock:^(NSString *gameSpin) {
        if ((int)self.viewModel.businessPoint >= (int)EXCHANGE_POINT_NUMBER && (int)self.viewModel.businessPoint > 0) {
            // 兑换游戏积分
        } else {
            [MBProgressHUD showSuccess:TLLocalizedString(business_point_is_not_enough) withFinishBlock:nil];
        }
    }];
    [alertView show];
}

- (void)onUnloaded:(NSString *)gameID {
    if (!self.isBack) {
        [self onGameQuit];
    }
    self.currentGameID = @"";
    [self.navigationController popViewControllerAnimated:YES];
}

- (void)onGameSoundPlay:(NSString *)name isPlay:(BOOL)isPlay url:(NSString *)url isLoop:(BOOL)isLoop volume:(int)volume {
    kLog(@"[onGameSoundVolumeChange] name：%@, isPlay：%d, url：%@, isLoop：%d, volume：%d", name, isPlay, url, isLoop, volume);
}

- (void)onGameSoundVolumeChange:(NSString *)name volume:(int)volume {
    kLog(@"[onGameSoundVolumeChange] name：%@, volume：%d",  name, volume);
}

- (void)onGameError:(int)errorCode errorMessage:(nonnull NSString *)errorMessage {
    kLog(@"[onGameError] errorCode：%d", errorCode);
    [MBProgressHUD showSuccess:[NSString stringWithFormat:TLLocalizedString(game_running_error), [ZegoConstants getErrorMsg:(int)errorCode]] withFinishBlock:nil];
}

- (void)onActionEventUpdate:(int)actionID data:(NSString *)data {
    kLog(@"[onActionEventUpdate] buttonID：%d, data：%@",  actionID, data);
    [MBProgressHUD showSuccess:[NSString stringWithFormat:@"onActionEventUpdate - buttonID:%d, data:%@", actionID, data] withFinishBlock:nil];
}

- (void)onGameResult:(NSString *)result {
    kLog(@"[onGameResult] result：%@", result);
    [MBProgressHUD showSuccess:[NSString stringWithFormat:@"onGameRoundResult - result：%@", result] withFinishBlock:nil];
}

- (ZegoGameRobotConfig *)onRobotConfigRequire:(NSString *)gameID {
    kLog(@"[onRobotConfigRequire] gameID：%@", gameID);
    NSString *name = [ZGUserInfoRandom getRandomName];
    NSString *avatar = [ZGUserInfoRandom getRandomAvatar];
    NSLog(@"onRobotConfigRequire name:%@  avatar:%@", name, avatar);
    ZegoGameRobotConfig *config = [[ZegoGameRobotConfig alloc] initWithRobotLevel:7 robotNickName:name robotAvatar:avatar useRobotAfterTime:10 minRobotPoints:1000 maxRobotPoints:5000];

    return config;
}


- (void)onGameActionRequest:(int)actionID data:(NSString *)data callBack:(ZegoGameActionCallback *)callBack {

}

- (void)onComponentListUpdate:(NSArray *)componentList {
    for (ZegoGameComponent *componentInfo in componentList) {
        // componentInfo.name 表示可控制的组件名称
        // 可继续调用 [ZegoMiniGameEngine.sharedEngine setGameComponent:isVisible:callBack:] 设置组件的隐藏与否
    }
}

- (void)handleDeviceOrientationChange {
    if ([UIApplication sharedApplication].statusBarOrientation == UIInterfaceOrientationPortrait || [UIApplication sharedApplication].statusBarOrientation == UIDeviceOrientationPortraitUpsideDown) {
            // 竖屏
            self.tabBarController.tabBar.hidden = YES;
         
        } else {
            // 横屏
            self.tabBarController.tabBar.hidden = YES;
        }
}

#pragma mark - getter

- (UILabel *)loadingLabel {
    if (!_loadingLabel) {
        _loadingLabel = [[UILabel alloc] init];
        _loadingLabel.textColor = [UIColor whiteColor];
        _loadingLabel.hidden = YES;
        _loadingLabel.font = BOLD_FONT(20);
        _loadingLabel.text = TLLocalizedString(loading_game);
        _loadingLabel.textAlignment = NSTextAlignmentCenter;
        [_loadingLabel sizeToFit];
    }
    return _loadingLabel;
}

- (UIButton *)closeGameBtn {
    if (!_closeGameBtn) {
        _closeGameBtn = [UIButton actionButtonWithTitle:TLLocalizedString(close_game)];
        _closeGameBtn.layer.cornerRadius = 18;
        _closeGameBtn.layer.masksToBounds = YES;
        _closeGameBtn.titleLabel.font = [UIFont systemFontOfSize:13];
        [_closeGameBtn addTarget:self action:@selector(onClickCloseGameAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _closeGameBtn;
}

- (UIView *)gameContainView {
    if (!_gameContainView) {
        _gameContainView = [[UIView alloc] init];
        _gameContainView.backgroundColor = [UIColor orangeColor];
    }
    return _gameContainView;
}

- (void)dealloc {
    NSLog(@"");
}

@end
