//
//  ZGLanguageSettingEntranceView.m
//  ZegoMiniGameEngineDemo
//
//  Created by zego on 2024/5/23.
//

#import "ZGLanguageSettingEntranceView.h"

@interface ZGLanguageSettingEntranceView ()

@property (nonatomic, strong) UILabel *titleLabel;
@property (nonatomic, strong) UILabel *languageLabel;
@property (nonatomic, strong) UIButton *iconRightBtn;
@property (nonatomic, copy) ZGRightButtonClickListener listener;

@end

@implementation ZGLanguageSettingEntranceView

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self setupUI];
    }
    
    return self;
}

- (void)setupUI {
    self.layer.cornerRadius = 10;
    self.layer.masksToBounds = YES;
    self.backgroundColor = [UIColor colorWithHexString:@"#574979"];
    
    [self addSubview:self.titleLabel];
    [self addSubview:self.languageLabel];
    [self addSubview:self.iconRightBtn];
    
    [self.titleLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self);
        make.left.equalTo(self).offset(17);
        make.bottom.equalTo(self.mas_bottom).offset(-14);
    }];
    
    [self.languageLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.titleLabel);
        make.right.equalTo(self).offset(-32);
    }];
    
    [self.iconRightBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.titleLabel);
        make.left.equalTo(self.languageLabel.mas_right).offset(2);
        make.width.height.mas_equalTo(20);
    }];
    
    UITapGestureRecognizer *tapGR = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(onRightActionButtonClick:)];
    [self addGestureRecognizer:tapGR];
}

- (void)updateLanguageLabel:(NSString *)text {
    self.languageLabel.text = text;
}

- (void)setOnRightButtonClick:(ZGRightButtonClickListener)listener {
    self.listener = listener;
}

- (void)onRightActionButtonClick:(UIGestureRecognizer *)tap {
    if (_listener != NULL) {
        self.listener();
    }
}

- (UILabel *)titleLabel {
    if(!_titleLabel) {
        _titleLabel = [[UILabel alloc] init];
        _titleLabel.text = TLLocalizedString(language_setting);
        _titleLabel.textColor = [UIColor whiteColor];
        _titleLabel.font = [UIFont systemFontOfSize:14];
        _titleLabel.textAlignment = NSTextAlignmentLeft;
        [_titleLabel sizeToFit];
    }
    return _titleLabel;
}

- (UILabel *)languageLabel {
    if(!_languageLabel) {
        _languageLabel = [[UILabel alloc] init];
        _languageLabel.text = TLLocalizedString(language_system);
        _languageLabel.textColor = [UIColor colorWithHexString:@"A8A4AE"];
        _languageLabel.font = [UIFont systemFontOfSize:14];
        _languageLabel.textAlignment = NSTextAlignmentLeft;
        [_languageLabel sizeToFit];
    }
    return _languageLabel;
}

- (UIButton *)iconRightBtn {
    if (!_iconRightBtn) {
        _iconRightBtn = [[UIButton alloc] init];
        _iconRightBtn.enabled = NO;
        [_iconRightBtn setImage:[UIImage imageNamed:@"icon_right.png"] forState:UIControlStateNormal];
    }
    return _iconRightBtn;
}

@end
