//
//  ZGSettingViewController.m
//  ZegoMiniGameEngineDemo
//
//  Created by zego on 2023/3/20.
//

#import "ZGSettingViewController.h"
#import "ZGSelectLanguageController.h"
#import "NSBundle+CLLanguage.h"
#import "ZGLanguageSettingEntranceView.h"
#import "ZGSDKVersionView.h"

@interface ZGSettingViewController ()

@property (nonatomic, strong) ZGLanguageSettingEntranceView *languageSettingEntranceView;
@property (nonatomic, assign) NSInteger selectIndex;

@property (nonatomic, strong) ZGSDKVersionView *gameSDKVersionView;

@end

@implementation ZGSettingViewController

- (void)viewDidLoad {
    [super viewDidLoad];
  
    self.view.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageNamed:@"img_bg"]];
    [self setupLanguageEntranceView];
    [self setupData];
    
    [self setupGameSDKVersionView];
}

- (void)setupData {
    if ([[NSBundle currentLanguage] isEqualToString:@"zh-Hans"]) {
        [self.languageSettingEntranceView updateLanguageLabel:TLLocalizedString(language_zh)];
        self.selectIndex = 1;
    } else if ([[NSBundle currentLanguage] isEqualToString:@"en"]) {
        [self.languageSettingEntranceView updateLanguageLabel:TLLocalizedString(language_en)];
        self.selectIndex = 2;
    } else {
        [self.languageSettingEntranceView updateLanguageLabel:TLLocalizedString(language_system)];
        self.selectIndex = 0;
    }
}

- (void)setupLanguageEntranceView {
    self.languageSettingEntranceView = [ZGLanguageSettingEntranceView new];
    [self.view addSubview:self.languageSettingEntranceView];
  
    [self.languageSettingEntranceView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.view).offset(kTLTopStatusBarHeight + 76);
        make.left.equalTo(self.view).offset(16);
        make.right.equalTo(self.view).offset(-16);
        make.height.mas_equalTo(48);
    }];
    
    [self.languageSettingEntranceView setOnRightButtonClick:^{
        @weakify(self)
        ZGSelectLanguageController *vc = [[ZGSelectLanguageController alloc] init];
        vc.selectBlock = ^(NSInteger index, NSString *title) {
            @strongify(self)
            self.selectIndex = index;
            [self.languageSettingEntranceView updateLanguageLabel:title];
        };
        vc.defaultIndex = self.selectIndex;
        [self.navigationController pushViewController:vc animated:YES];
    }];
}

- (void)setupGameSDKVersionView {
    self.gameSDKVersionView = [ZGSDKVersionView new];
    [self.view addSubview:self.gameSDKVersionView];
        
    [self.gameSDKVersionView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.languageSettingEntranceView.mas_bottom).offset(20);
        make.left.equalTo(self.view).offset(16);
        make.right.equalTo(self.view).offset(-16);
        make.height.mas_equalTo(48);
    }];
  
    [self.gameSDKVersionView updateVersionLabel:[ZegoMiniGameEngine getVersion]];
}

@end
