//
//  GameTableViewCell.m
//  ZegoMiniGameEngineDemo
//
//  Created by zego on 2023/2/3.
//

#import "GameTableViewCell.h"
#import "DemoGameInfo.h"

@interface GameTableViewCell ()

@property (nonatomic, strong) UIImageView *gameImageView;
@property (nonatomic, strong) UILabel *nameLabel;
@property (nonatomic, strong) UIButton *exchangeButton;
@property (nonatomic, strong) UIButton *progressButton;
@property (nonatomic, strong) UIButton *cancelButton;
@property (nonatomic, strong) DemoGameInfo *gameInfo;
@end

@implementation GameTableViewCell

- (instancetype)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier{
    self = [super initWithStyle:style reuseIdentifier:reuseIdentifier];
    if(self){
        [self createUI];
    }
    return self;
}

- (void)createUI {
    [self addSubview:self.gameImageView];
    [self addSubview:self.nameLabel];
    [self addSubview:self.gamePointsAmountLabel];
    [self.contentView addSubview:self.exchangeButton];
    [self.contentView addSubview:self.progressButton];
    [self.contentView addSubview:self.cancelButton];
    
    [self.gameImageView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self).offset(10);
        make.bottom.equalTo(self).offset(-8);
        make.left.mas_equalTo(20);
        make.width.height.mas_equalTo(60);
    }];
    
    [self.nameLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.gameImageView.mas_top);
        make.left.equalTo(self.gameImageView.mas_right).offset(18);
        make.height.mas_equalTo(20);
    }];
    
    [self.gamePointsAmountLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.nameLabel.mas_right).offset(5);
//        make.centerX.equalTo(self);
        make.top.mas_equalTo(self.nameLabel.mas_top);
        make.centerY.equalTo(self.nameLabel.mas_centerY);
        make.height.mas_equalTo(20);
    }];
    
    [self.exchangeButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(self.mas_right).offset(-20);
//        make.centerY.equalTo(self.gameImageView.mas_centerY);
        make.top.mas_equalTo(self.contentView.mas_top).offset(5);
        make.width.mas_equalTo(120);
        make.height.mas_equalTo(30);
    }];
    
    [self.progressButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(self.mas_right).offset(-20);
//        make.centerY.equalTo(self.gameImageView.mas_centerY);
        make.bottom.mas_equalTo(self.contentView.mas_bottom).offset(-5);
        make.width.mas_equalTo(120);
        make.height.mas_equalTo(30);
    }];
    
    [self.cancelButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(self.progressButton.mas_left).offset(-20);
//        make.centerY.equalTo(self.gameImageView.mas_centerY);
        make.bottom.mas_equalTo(self.contentView.mas_bottom).offset(-5);
        make.width.mas_equalTo(80);
        make.height.mas_equalTo(30);
    }];
    
}

- (void)setMiniGameInfo:(DemoGameInfo *)gameInfo {
    _gameInfo = gameInfo;
    [self.gameImageView setImageWithURL:[NSURL URLWithString:gameInfo.gameInfo.thumbnail] placeholder:[UIImage imageNamed:@""]];
    self.nameLabel.text = gameInfo.gameInfo.gameName;
    self.gamePointsAmountLabel.text = [NSString stringWithFormat:@"%lu", gameInfo.gamePoint];
    [self.progressButton setTitle:[NSString stringWithFormat:@"preload:%d", gameInfo.progress] forState:UIControlStateNormal];
}

#pragma mark - action
- (void)onExchangeButtonActionButtonClick:(UIButton *)sender {
    NSLog(@"%@：兑换了10000积分", self.nameLabel.text);
    if (self.exchangeBlock) {
        self.exchangeBlock(self.gameInfo);
    }
}

- (void)preloadButtonClick:(UIButton *)sender {
    if (self.preloadBlock) {
        self.preloadBlock(self.gameInfo, sender);
    }
}

- (void)cancelButtonClick:(UIButton *)sender {
    if (self.cancelBlock) {
        self.cancelBlock(self.gameInfo, sender);
    }
}

- (void)longTapAction {
    if (self.longTapBlock) {
        self.longTapBlock(self.gameInfo);
    }
}

#pragma mark - getter
- (UIImageView *)gameImageView {
    if (!_gameImageView) {
        _gameImageView = [[UIImageView alloc] init];
        _gameImageView.userInteractionEnabled = true;
        UILongPressGestureRecognizer *longTap = [[UILongPressGestureRecognizer alloc] initWithTarget:self action:@selector(longTapAction)];
        [_gameImageView addGestureRecognizer:longTap];
        _gameImageView.contentMode = UIViewContentModeScaleAspectFill;
        _gameImageView.backgroundColor = [UIColor whiteColor];
        _gameImageView.layer.cornerRadius = 8.0f;
        _gameImageView.layer.masksToBounds = YES;
    }
    return _gameImageView;
}

- (UILabel *)nameLabel {
    if(!_nameLabel) {
        _nameLabel = [[UILabel alloc] init];
        _nameLabel.textColor = [UIColor whiteColor];
        _nameLabel.font = [UIFont systemFontOfSize:12];
        _nameLabel.textAlignment = NSTextAlignmentCenter;
        [_nameLabel sizeToFit];
    }
    return _nameLabel;
}

- (UILabel *)gamePointsAmountLabel {
    if (!_gamePointsAmountLabel) {
        _gamePointsAmountLabel = [[UILabel alloc] init];
        _gamePointsAmountLabel.text = @"10000";
        _gamePointsAmountLabel.textColor = [UIColor whiteColor];
        _gamePointsAmountLabel.font = [UIFont systemFontOfSize:12];
        _gamePointsAmountLabel.textAlignment = NSTextAlignmentCenter;
        [_gamePointsAmountLabel sizeToFit];
    }
    return _gamePointsAmountLabel;
}

- (UIButton *)exchangeButton {
    if (!_exchangeButton) {
        _exchangeButton = [UIButton actionButtonWithTitle:[NSString stringWithFormat: TLLocalizedString(exchange_x_game_point), 10000]];
        _exchangeButton.titleLabel.font = [UIFont systemFontOfSize:13];
        _exchangeButton.layer.cornerRadius = 16;
        _exchangeButton.layer.masksToBounds = YES;
        [_exchangeButton setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
        [_exchangeButton addTarget:self action:@selector(onExchangeButtonActionButtonClick:) forControlEvents:UIControlEventTouchUpInside];
        _exchangeButton.userInteractionEnabled = YES;
    }
    return _exchangeButton;
}

- (UIButton *)progressButton {
    if (!_progressButton) {
        _progressButton = [UIButton actionButtonWithTitle:@"preload:0"];
        _progressButton.titleLabel.font = [UIFont systemFontOfSize:13];
        _progressButton.layer.cornerRadius = 16;
        _progressButton.layer.masksToBounds = YES;
        [_progressButton setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
        [_progressButton addTarget:self action:@selector(preloadButtonClick:) forControlEvents:UIControlEventTouchUpInside];
        _progressButton.userInteractionEnabled = YES;
    }
    return _progressButton;
}

- (UIButton *)cancelButton {
    if (!_cancelButton) {
        _cancelButton = [UIButton actionButtonWithTitle:@"cancel"];
        _cancelButton.titleLabel.font = [UIFont systemFontOfSize:13];
        _cancelButton.layer.cornerRadius = 16;
        _cancelButton.layer.masksToBounds = YES;
        [_cancelButton setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
        [_cancelButton addTarget:self action:@selector(cancelButtonClick:) forControlEvents:UIControlEventTouchUpInside];
        _cancelButton.userInteractionEnabled = YES;
    }
    return _cancelButton;
}
@end
